@@ -1,17 +1,17 @@
-# $Id: FollowTail.pm 2267 2008-01-11 15:50:02Z bingosnet $
+# $Id: FollowTail.pm,v 1.49 2004/01/28 23:19:11 rcaputo Exp $
 
 package POE::Wheel::FollowTail;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2267 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = do {my@r=(q$Revision: 1.49 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
-use Carp qw( croak carp );
-use Symbol qw( gensym );
-use POSIX qw(SEEK_SET SEEK_CUR SEEK_END S_ISCHR S_ISBLK);
+use Carp;
+use Symbol;
+use POSIX qw(SEEK_SET SEEK_CUR SEEK_END);
 use POE qw(Wheel Driver::SysRW Filter::Line);
-use IO::Handle ();
+use IO::Handle;
 
 sub CRIMSON_SCOPE_HACK ($) { 0 }
 
@@ -58,10 +58,20 @@ sub new {
   my %params = @_;
 
   croak "wheels no longer require a kernel reference as their first parameter"
-    if @_ and (ref($_[0]) eq 'POE::Kernel');
+    if (@_ && (ref($_[0]) eq 'POE::Kernel'));
 
   croak "$type requires a working Kernel" unless (defined $poe_kernel);
 
+  # STATE-EVENT
+  if (exists $params{InputState}) {
+    croak "InputState is deprecated.  Use InputEvent";
+  }
+
+  # STATE-EVENT
+  if (exists $params{ErrorState}) {
+    croak "ErrorState is deprecated.  Use ErrorEvent";
+  }
+
   croak "FollowTail requires a Handle or Filename parameter, but not both"
     unless $params{Handle} xor defined $params{Filename};
 
@@ -76,12 +86,25 @@ sub new {
   my $handle   = $params{Handle};
   my $filename = $params{Filename};
 
-  if (defined $filename) {
-    $handle = _open_file($filename);
-  }
-
   my @start_stat;
-  @start_stat = stat($filename) if defined $handle;
+  if (defined $filename) {
+    $handle = gensym();
+
+    # FIFOs (named pipes) are opened R/W so they don't report EOF.
+    # TODO Make this nonfatal, in case the file doesn't exist but will
+    # later.  For example, the file may be caught in the middle of a
+    # rotation.
+    if (-p $filename) {
+      open $handle, "+<$filename" or
+        croak "can't open fifo $filename for R/W: $!";
+    }
+
+    # Everything else is opened read-only.
+    else {
+      open $handle, "<$filename" or croak "can't open $filename: $!";
+    }
+    @start_stat = stat($filename);
+  }
 
   my $poll_interval = (
     defined($params{PollInterval})
@@ -103,48 +126,84 @@ sub new {
     $seek = -4096;
   }
 
-  my $self = bless [
-    $handle,                          # SELF_HANDLE
-    $filename,                        # SELF_FILENAME
-    $driver,                          # SELF_DRIVER
-    $filter,                          # SELF_FILTER
-    $poll_interval,                   # SELF_INTERVAL
-    delete $params{InputEvent},       # SELF_EVENT_INPUT
-    delete $params{ErrorEvent},       # SELF_EVENT_ERROR
-    delete $params{ResetEvent},       # SELF_EVENT_RESET
-    &POE::Wheel::allocate_wheel_id(), # SELF_UNIQUE_ID
-    undef,                            # SELF_STATE_READ
-    \@start_stat,                     # SELF_LAST_STAT
-    undef,                            # SELF_FOLLOW_MODE
-  ], $type;
+  my $self = bless
+    [ $handle,                          # SELF_HANDLE
+      $filename,                        # SELF_FILENAME
+      $driver,                          # SELF_DRIVER
+      $filter,                          # SELF_FILTER
+      $poll_interval,                   # SELF_INTERVAL
+      delete $params{InputEvent},       # SELF_EVENT_INPUT
+      delete $params{ErrorEvent},       # SELF_EVENT_ERROR
+      delete $params{ResetEvent},       # SELF_EVENT_RESET
+      &POE::Wheel::allocate_wheel_id(), # SELF_UNIQUE_ID
+      undef,                            # SELF_STATE_READ
+      \@start_stat,                     # SELF_LAST_STAT
+      undef,                            # SELF_FOLLOW_MODE
+    ], $type;
 
-  # We couldn't open a file.  SeekBack won't be used because it
-  # assumes the file already exists.  If you need more complex
-  # seeking, consider opening and seeking yourself.  In fact, the
-  # whole SeekBack concept should be deprecated as it only opens a can
-  # full of arbitrarily complex worms.  Maybe later.
+  # SeekBack and partial-input discarding only work for plain files.
+  # SeekBack attempts to position the file pointer somewhere before
+  # the end of the file.  If it's specified, we assume the user knows
+  # where a record begins.  Otherwise we just seek back and discard
+  # everything to EOF so we can frame the input record.
 
-  unless (defined $handle) {
-    carp "FollowTail does not support SeekBack on nonexistent files"
-      if defined $params{SeekBack};
+  if (-f $handle) {
+    my $end = sysseek($handle, 0, SEEK_END);
 
+    # Seeking back from EOF.
+    if ($seek < 0) {
+      if (defined($end) and ($end < -$seek)) {
+        sysseek($handle, 0, SEEK_SET);
+      }
+      else {
+        sysseek($handle, $seek, SEEK_END);
+      }
+    }
+
+    # Seeking forward from the beginning of the file.
+    elsif ($seek > 0) {
+      if ($seek > $end) {
+        sysseek($handle, 0, SEEK_END);
+      }
+      else {
+        sysseek($handle, $seek, SEEK_SET);
+      }
+    }
+
+    # If they set Seek to 0, we start at the beginning of the file.
+    # If it was SeekBack, we start at the end.
+    elsif (exists $params{Seek}) {
+      sysseek($handle, 0, SEEK_SET);
+    }
+    elsif (exists $params{SeekBack}) {
+      sysseek($handle, 0, SEEK_END);
+    }
+    else {
+      die;
+    }
+
+    # Discard partial input chunks unless a SeekBack was specified.
+    unless (defined $params{SeekBack} or defined $params{Seek}) {
+      while (defined(my $raw_input = $driver->get($handle))) {
+        # Skip out if there's no more input.
+        last unless @$raw_input;
+        $filter->get($raw_input);
+      }
+    }
+
+    # Start the timer loop.
     $self->[SELF_FOLLOW_MODE] = MODE_TIMER;
     $self->_define_timer_states();
-
-    return $self;
   }
 
   # Strange things that ought not be tailed?  Directories...
-
-  if (-d $handle) {
-    croak "FollowTail does not tail directories";
+  elsif (-d $handle) {
+    croak "FollowTail does not accept directories";
   }
 
-  # SeekBack only works with plain files.  We won't honor SeekBack,
-  # and we will use select_read to watch the handle rather than the
-  # polling interval.
-
-  unless (-f $handle) {
+  # Otherwise it's not a plain file.  We won't honor SeekBack, and we
+  # will use select_read to watch the handle.
+  else {
     carp "FollowTail does not support SeekBack on a special file"
       if defined $params{SeekBack};
     carp "FollowTail does not need PollInterval for special files"
@@ -153,66 +212,8 @@ sub new {
     # Start the select loop.
     $self->[SELF_FOLLOW_MODE] = MODE_SELECT;
     $self->_define_select_states();
-
-    return $self;
   }
 
-  # We only get this far with plain files that have successfully been
-  # opened at the time the wheel is created.  SeekBack and
-  # partial-input discarding work here.
-  #
-  # SeekBack attempts to position the file pointer somewhere before
-  # the end of the file.  If it's specified, we assume the user knows
-  # where a record begins.  Otherwise we just seek back and discard
-  # everything to EOF so we can frame the input record.
-
-  my $end = sysseek($handle, 0, SEEK_END);
-
-  # Seeking back from EOF.
-  if ($seek < 0) {
-    if (defined($end) and ($end < -$seek)) {
-      sysseek($handle, 0, SEEK_SET);
-    }
-    else {
-      sysseek($handle, $seek, SEEK_END);
-    }
-  }
-
-  # Seeking forward from the beginning of the file.
-  elsif ($seek > 0) {
-    if ($seek > $end) {
-      sysseek($handle, 0, SEEK_END);
-    }
-    else {
-      sysseek($handle, $seek, SEEK_SET);
-    }
-  }
-
-  # If they set Seek to 0, we start at the beginning of the file.
-  # If it was SeekBack, we start at the end.
-  elsif (exists $params{Seek}) {
-    sysseek($handle, 0, SEEK_SET);
-  }
-  elsif (exists $params{SeekBack}) {
-    sysseek($handle, 0, SEEK_END);
-  }
-  else {
-    die;  # Should never happen.
-  }
-
-  # Discard partial input chunks unless a SeekBack was specified.
-  unless (defined $params{SeekBack} or defined $params{Seek}) {
-    while (defined(my $raw_input = $driver->get($handle))) {
-      # Skip out if there's no more input.
-      last unless @$raw_input;
-      $filter->get($raw_input);
-    }
-  }
-
-  # Start the timer loop.
-  $self->[SELF_FOLLOW_MODE] = MODE_TIMER;
-  $self->_define_timer_states();
-
   return $self;
 }
 
@@ -232,48 +233,48 @@ sub _define_select_states {
   my $event_error = \$self->[SELF_EVENT_ERROR];
   my $event_reset = \$self->[SELF_EVENT_RESET];
 
-  TRACE_POLL and warn "<poll> defining select state";
+  TRACE_POLL and warn "defining select state";
 
-  $poe_kernel->state(
-    $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> select read",
-    sub {
+  $poe_kernel->state
+    ( $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> select read",
+      sub {
 
-      # Protects against coredump on older perls.
-      0 && CRIMSON_SCOPE_HACK('<');
+        # Protects against coredump on older perls.
+        0 && CRIMSON_SCOPE_HACK('<');
 
-      # The actual code starts here.
-      my ($k, $ses) = @_[KERNEL, SESSION];
+        # The actual code starts here.
+        my ($k, $ses) = @_[KERNEL, SESSION];
 
-      eval {
-        sysseek($handle, 0, SEEK_CUR);
-      };
-      $! = 0;
+        eval {
+          sysseek($handle, 0, SEEK_CUR);
+        };
+        $! = 0;
 
-      TRACE_POLL and warn "<poll> " . time . " read ok";
+        TRACE_POLL and warn time . " read ok";
 
-      if (defined(my $raw_input = $driver->get($handle))) {
-        if (@$raw_input) {
-          TRACE_POLL and warn "<poll> " . time . " raw input";
-          foreach my $cooked_input (@{$filter->get($raw_input)}) {
-            TRACE_POLL and warn "<poll> " . time . " cooked input";
-            $k->call($ses, $$event_input, $cooked_input, $unique_id);
+        if (defined(my $raw_input = $driver->get($handle))) {
+          if (@$raw_input) {
+            TRACE_POLL and warn time . " raw input";
+            foreach my $cooked_input (@{$filter->get($raw_input)}) {
+              TRACE_POLL and warn time . " cooked input";
+              $k->call($ses, $$event_input, $cooked_input, $unique_id);
+            }
           }
         }
-      }
 
-      # Error reading.  Report the error if it's not EOF, or if it's
-      # EOF on a socket or TTY.  Shut down the select, too.
-      else {
-        if ($! or (-S $handle) or (-t $handle)) {
-          TRACE_POLL and warn "<poll> " . time . " error: $!";
-          $$event_error and
-            $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
-          $k->select($handle);
+        # Error reading.  Report the error if it's not EOF, or if it's
+        # EOF on a socket or TTY.  Shut down the select, too.
+        else {
+          if ($! or (-S $handle) or (-t $handle)) {
+            TRACE_POLL and warn time . " error: $!";
+            $$event_error and
+              $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
+            $k->select($handle);
+          }
+          eval { IO::Handle::clearerr($handle) }; # could be a globref
         }
-        eval { IO::Handle::clearerr($handle) }; # could be a globref
       }
-    }
-  );
+    );
 
   $poe_kernel->select_read($handle, $self->[SELF_STATE_READ]);
 }
@@ -290,147 +291,119 @@ sub _define_timer_states {
   my $poll_interval = $self->[SELF_INTERVAL];
   my $filename      = $self->[SELF_FILENAME];
   my $last_stat     = $self->[SELF_LAST_STAT];
+  my $handle        = $self->[SELF_HANDLE];
+  my $state_read    = $self->[SELF_STATE_READ] =
+    ref($self) . "($unique_id) -> timer read";
 
-  my $handle        = \$self->[SELF_HANDLE];
   my $event_input   = \$self->[SELF_EVENT_INPUT];
   my $event_error   = \$self->[SELF_EVENT_ERROR];
   my $event_reset   = \$self->[SELF_EVENT_RESET];
 
-  $self->[SELF_STATE_READ] = ref($self) . "($unique_id) -> timer read";
-  my $state_read    = \$self->[SELF_STATE_READ];
+  TRACE_POLL and warn "defining timer state";
 
-  TRACE_POLL and warn "<poll> defining timer state";
+  $poe_kernel->state
+    ( $state_read,
+      sub {
 
-  $poe_kernel->state(
-    $$state_read,
-    sub {
+        # Protects against coredump on older perls.
+        0 && CRIMSON_SCOPE_HACK('<');
 
-      # Protects against coredump on older perls.
-      0 && CRIMSON_SCOPE_HACK('<');
+        # The actual code starts here.
+        my ($k, $ses) = @_[KERNEL, SESSION];
 
-      # The actual code starts here.
-      my ($k, $ses) = @_[KERNEL, SESSION];
+        eval {
+          if (defined $filename) {
+            my @new_stat = stat($filename);
 
-      eval {
-        if (defined $filename) {
-          my @new_stat = stat($filename);
+            TRACE_STAT_VERBOSE and do {
+              my @test_new = @new_stat;   splice(@test_new, 8, 1, "(removed)");
+              my @test_old = @$last_stat; splice(@test_old, 8, 1, "(removed)");
+              warn "=== @test_new" if "@test_new" ne "@test_old";
+            };
 
-          TRACE_STAT_VERBOSE and do {
-            my @test_new = @new_stat;   splice(@test_new, 8, 1, "(removed)");
-            my @test_old = @$last_stat; splice(@test_old, 8, 1, "(removed)");
-            warn "<stat> @test_new" if "@test_new" ne "@test_old";
-          };
+            if (@new_stat) {
 
-          if (@new_stat) {
-            my $did_reset;
-
-            # File shrank.  Consider it a reset.  Seek to the top of
-            # the file.
-            if ($new_stat[7] < $last_stat->[7]) {
-              $did_reset = 1;
-            }
-
-            $last_stat->[7] = $new_stat[7];
-
-            # Ignore rdev changes for non-device files
-            if (!S_ISBLK($new_stat[2]) and !S_ISCHR($new_stat[2])) {
-              $last_stat->[6] = $new_stat[6];
-            }
-
-            # Something fundamental about the file changed.  Reopen it.
-            if (
-              $new_stat[1] != $last_stat->[1] or # inode's number
-              $new_stat[0] != $last_stat->[0] or # inode's device
-              $new_stat[6] != $last_stat->[6] or # device type
-              $new_stat[3] != $last_stat->[3]    # number of links
-            ) {
-
-              TRACE_STAT and do {
-                warn "<stat> inode $new_stat[1] != old $last_stat->[1]\n"
-                  if $new_stat[1] != $last_stat->[1];
-                warn "<stat> inode device $new_stat[0] != old $last_stat->[0]\n"
-                  if $new_stat[0] != $last_stat->[0];
-                warn "<stat> device type $new_stat[6] != old $last_stat->[6]\n"
-                  if $new_stat[6] != $last_stat->[6];
-                warn "<stat> link count $new_stat[3] != old $last_stat->[3]\n"
-                  if $new_stat[3] != $last_stat->[3];
-                warn "<stat> file size $new_stat[7] < old $last_stat->[7]\n"
-                  if $new_stat[7] < $last_stat->[7];
-              };
-
-              # The file may have rolled.  Try one more read before moving on.
-              if (
-                defined $$handle and
-                defined(my $raw_input = $driver->get($$handle))
-              ) {
-
-                # First read the remainder of the file.
-                # Got input.  Read a bunch of it, then poll again right away.
-                if (@$raw_input) {
-                  TRACE_POLL and warn "<poll> " . time . " raw input\n";
-                  foreach my $cooked_input (@{$filter->get($raw_input)}) {
-                    TRACE_POLL and warn "<poll> " . time . " cooked input\n";
-                    $k->call($ses, $$event_input, $cooked_input, $unique_id);
-                  }
-                }
-                $k->yield($$state_read) if defined $$state_read;
+              # File shrank.  Consider it a reset.  Seek to the top of
+              # the file.
+              if ($new_stat[7] < $last_stat->[7]) {
+                $$event_reset and $k->call($ses, $$event_reset, $unique_id);
+                sysseek($handle, 0, SEEK_SET);
               }
 
-              @$last_stat = @new_stat;
-              close $$handle if defined $$handle;
-              $$handle = _open_file($filename);
+              $last_stat->[7] = $new_stat[7];
 
-              $did_reset = 1;
-            }
+              # Something fundamental about the file changed.  Reopen it.
+              if ( $new_stat[1] != $last_stat->[1] or # inode's number
+                   $new_stat[0] != $last_stat->[0] or # inode's device
+                   $new_stat[6] != $last_stat->[6] or # device type
+                   $new_stat[3] != $last_stat->[3]    # number of links
+                 ) {
 
-            if ($did_reset) {
-              $$event_reset and $k->call($ses, $$event_reset, $unique_id);
-              sysseek($$handle, 0, SEEK_SET);
+                TRACE_STAT and do {
+                  warn "inode $new_stat[1] != old $last_stat->[1]\n"
+                    if $new_stat[1] != $last_stat->[1];
+                  warn "inode device $new_stat[0] != old $last_stat->[0]\n"
+                    if $new_stat[0] != $last_stat->[0];
+                  warn "device type $new_stat[6] != old $last_stat->[6]\n"
+                    if $new_stat[6] != $last_stat->[6];
+                  warn "number of links $new_stat[3] != old $last_stat->[3]\n"
+                    if $new_stat[3] != $last_stat->[3];
+                  warn "file size $new_stat[7] < old $last_stat->[7]\n"
+                    if $new_stat[7] < $last_stat->[7];
+                };
+
+                @$last_stat = @new_stat;
+
+                close $handle;
+                unless (open $handle, "<$filename") {
+                  $$event_error and
+                    $k->call( $ses, $$event_error, 'reopen',
+                              ($!+0), $!, $unique_id
+                            );
+                }
+              }
             }
           }
-        }
-      };
+        };
+        $! = 0;
 
-      $! = 0;
-      TRACE_POLL and warn "<poll> " . time . " read ok\n";
+        TRACE_POLL and warn time . " read ok\n";
 
-      # No open file.  Go around again.
-      unless (defined $$handle) {
-        $k->delay($$state_read, $poll_interval) if defined $$state_read;
-      }
-
-      # Got input.  Read a bunch of it, then poll again right away.
-      elsif (defined(my $raw_input = $driver->get($$handle))) {
-        if (@$raw_input) {
-          TRACE_POLL and warn "<poll> " . time . " raw input\n";
-          foreach my $cooked_input (@{$filter->get($raw_input)}) {
-            TRACE_POLL and warn "<poll> " . time . " cooked input\n";
-            $k->call($ses, $$event_input, $cooked_input, $unique_id);
+        # Got input.  Read a bunch of it, then poll again right away.
+        if (defined(my $raw_input = $driver->get($handle))) {
+          if (@$raw_input) {
+            TRACE_POLL and warn time . " raw input\n";
+            foreach my $cooked_input (@{$filter->get($raw_input)}) {
+              TRACE_POLL and warn time . " cooked input\n";
+              $k->call($ses, $$event_input, $cooked_input, $unique_id);
+            }
           }
+          $k->yield($state_read);
         }
-        $k->yield($$state_read) if defined $$state_read;
-      }
 
-      # Got an error of some sort.
-      else {
-        TRACE_POLL and warn "<poll> " . time . " set delay\n";
-        if ($!) {
-          TRACE_POLL and warn "<poll> " . time . " error: $!\n";
-          $$event_error and
-            $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
-          $k->select($$handle);
+        # Got an error of some sort.
+        else {
+          TRACE_POLL and warn time . " set delay\n";
+          if ($!) {
+            TRACE_POLL and warn time . " error: $!\n";
+            $$event_error and
+              $k->call($ses, $$event_error, 'read', ($!+0), $!, $unique_id);
+            $k->select($handle);
+          }
+          $k->delay($state_read, $poll_interval);
+          IO::Handle::clearerr($handle);
         }
-        $k->delay($$state_read, $poll_interval) if defined $$state_read;
-        IO::Handle::clearerr($$handle);
       }
-    }
-  );
+    );
 
-  # Fire up the loop.  The delay() aspect of the loop will prevent
-  # duplicate events from being significant for long.
-  $poe_kernel->delay($$state_read, 0);
+  $poe_kernel->yield($state_read);
 }
 
+# ### Define the select states, and begin reading the special handle.
+# ### This also relies on stupid closure tricks.
+
+#     $poe_kernel->select($handle, $self->[SELF_STATE_READ]);
+
 #------------------------------------------------------------------------------
 
 sub event {
@@ -440,6 +413,11 @@ sub event {
   while (@_) {
     my ($name, $event) = splice(@_, 0, 2);
 
+    # STATE-EVENT
+    if ($name =~ /^(.*?)State$/) {
+      croak "$name is deprecated.  Use $1Event";
+    }
+
     if ($name eq 'InputEvent') {
       if (defined $event) {
         $self->[SELF_EVENT_INPUT] = $event;
@@ -458,6 +436,16 @@ sub event {
       carp "ignoring unknown FollowTail parameter '$name'";
     }
   }
+
+  if ($self->[SELF_FOLLOW_MODE] & MODE_TIMER) {
+    $self->_define_timer_states();
+  }
+  elsif ($self->[SELF_FOLLOW_MODE] & MODE_SELECT) {
+    $self->_define_select_states();
+  }
+  else {
+    die;
+  }
 }
 
 #------------------------------------------------------------------------------
@@ -466,8 +454,7 @@ sub DESTROY {
   my $self = shift;
 
   # Remove our tentacles from our owner.
-  $poe_kernel->select($self->[SELF_HANDLE]) if defined $self->[SELF_HANDLE];
-
+  $poe_kernel->select($self->[SELF_HANDLE]);
   $poe_kernel->delay($self->[SELF_STATE_READ]);
 
   if ($self->[SELF_STATE_READ]) {
@@ -484,28 +471,6 @@ sub ID {
   return $_[0]->[SELF_UNIQUE_ID];
 }
 
-sub tell {
-  my $self = shift;
-  return sysseek($self->[SELF_HANDLE], 0, SEEK_CUR);
-}
-
-sub _open_file {
-  my $filename = shift;
-
-  my $handle = gensym();
-
-  # FIFOs (named pipes) are opened R/W so they don't report EOF.
-  # Everything else is opened read-only.
-  if (-p $filename) {
-    return unless open($handle, "+<$filename");
-  }
-  else {
-    return unless open($handle, "<$filename");
-  }
-
-  return $handle;
-}
-
 ###############################################################################
 1;
 
@@ -541,8 +506,6 @@ POE::Wheel::FollowTail - follow the tail of an ever-growing file
     Seek         => $offset,            # How far from BOF to start
   );
 
-  $pos = $wheel->tell();  # Get the current log position.
-
 =head1 DESCRIPTION
 
 FollowTail follows the end of an ever-growing file, such as a log of
@@ -551,16 +514,6 @@ appended to its file.
 
 This is a read-only wheel so it does not include a put() method.
 
-=head1 CONSTRUCTOR
-
-=over 
-
-=item new
-
-new() creates a new wheel, returning the wheels reference.
-
-=back
-
 =head1 PUBLIC METHODS
 
 =over 2
@@ -578,18 +531,6 @@ The ID method returns a FollowTail wheel's unique ID.  This ID will be
 included in every event the wheel generates, and it can be used to
 match events with the wheels which generated them.
 
-=item tell
-
-Returns where POE::Wheel::FollowTail is currently in the log file.
-tell() may be useful for seeking back into a file when resuming
-tailing.
-
-  my $pos = $wheel->tell();
-
-FollowTail generally reads ahead of the data it returns, so the file
-position is not necessarily the point after the last record you have
-received.
-
 =back
 
 =head1 EVENTS AND PARAMETERS
@@ -648,7 +589,7 @@ time.  Please be careful.
 The SeekBack parameter tells FollowTail how far back from the end of
 the file to start reading.  Its value is specified in bytes, and
 values greater than the file's current size will quietly cause
-FollowTail to start from the file's beginning.
+FollowTail to ltart from the file's beginning.
 
 A SeekBack parameter of 0 starts FollowTail at the end of the file.
 It's recommended to omit Seek and SeekBack to start from the end of a
@@ -679,8 +620,6 @@ FollowTail can watch a file or device that's already open.  Give it
 the open filehandle with its Handle parameter.
 
 FollowTail can watch a file by name, given as the Filename parameter.
-The named file does not need to exist.  FollowTail will wait for it to
-appear.
 
 This wheel can detect files that have been "reset".  That is, it can
 tell when log files have been restarted due to a rotation or purge.
@@ -747,11 +686,18 @@ the entire POE distribution.
 
 This wheel can't tail pipes and consoles on some systems.
 
+Because this wheel is cooperatively multitasked, it may lose records
+just prior to a file reset.  For a more robust way to watch files,
+consider using POE::Wheel::Run and your operating system's native
+"tail" utility instead.
+
+  $heap->{tail} = POE::Wheel::Run->new
+    ( Program     => [ "/usr/bin/tail", "-f", $file_name ],
+      StdoutEvent => "log_record",
+    );
+
 =head1 AUTHORS & COPYRIGHTS
 
 Please see L<POE> for more information about authors and contributors.
 
 =cut
-
-# rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
